/*
 * ##################################################
 * #                                                #
 * # rkDoubleSort                                   #
 * # Demo Java app for sorting double values        #
 * #                                                #
 * # Copyright (C) by René Knipschild               #
 * #                                                #
 * #                                email@rkcsd.com #
 * #                         www.reneknipschild.net #
 * #                                                #
 * ##################################################
 *
 * File: UserInterface.java - gui
 * Version: 1.1.0
 * Last modified: 2011/04/18 15:02 CEST
 * Author: rkCSD (go.reneknipschild.net/com)
 *
 * ===Notes==========================================
 * There are currently no notes.
 * ==================================================
 */

package com.rkcsd.apps.desktoptools.doublesort.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.rkcsd.apps.desktoptools.doublesort.tools.Browser;
import com.rkcsd.apps.desktoptools.doublesort.tools.DoubleSorter;

public class UserInterface {
	private JFrame window;

	private JPanel frame;

	private JLabel countLabel;

	private JSpinner count;

	private JPanel valuesPanel0;

	private JPanel valuesPanel1;

	private JPanel valuesPanel2;

	private JSpinner[] values;

	private ButtonGroup sortDirection;

	private JLabel sortDirectionLabel;

	private JRadioButton sortDirectionAscending;

	private JRadioButton sortDirectionDescending;

	private JPanel resultPanel;

	private JList result;

	private JButton sort;
	
	private JLabel copyright;

	public UserInterface() {
		this.window = new JFrame("rkDoubleSort");
		this.window.setIconImage(Toolkit.getDefaultToolkit().getImage(ClassLoader.getSystemResource("com/rkcsd/apps/desktoptools/doublesort/resources/rkIcon.png")));
		this.window.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		this.frame = new JPanel(new BorderLayout());
		this.frame.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		ComfortGridBagLayout layout = new ComfortGridBagLayout();
		this.frame.setLayout(layout);

		this.countLabel = new JLabel("Number of Double values:");
		this.count = new JSpinner(new SpinnerNumberModel(0, 0, 999, 1));
		this.valuesPanel0 = new JPanel(new BorderLayout());
		this.valuesPanel1 = new JPanel(new BorderLayout());
		this.valuesPanel2 = new JPanel(new GridLayout());
		this.values = new JSpinner[3];
		this.sortDirection = new ButtonGroup();
		this.sortDirectionLabel = new JLabel("Sort order:");
		this.sortDirectionAscending = new JRadioButton("Ascending");
		this.sortDirectionDescending = new JRadioButton("Descending");
		this.resultPanel = new JPanel(new BorderLayout());
		this.result = new JList();
		this.sort = new JButton("Sort");
		this.copyright = new JLabel("<html><p>Copyright © 2009 René Knipschild (<a href=\"http://www.reneknipschild.net/\">www.reneknipschild.net</a>)</p></html>)");

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				UserInterface.this.count.requestFocusInWindow();
			}
		});
		this.sortDirectionAscending.setSelected(true);
		this.valuesPanel0.setPreferredSize(new Dimension(this.valuesPanel0.getPreferredSize().width, 50));
		this.valuesPanel0.add(new JScrollPane(this.valuesPanel1, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		this.resultPanel.setPreferredSize(new Dimension(this.resultPanel.getPreferredSize().width, 50));
		this.resultPanel.add(new JScrollPane(this.result, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		this.copyright.setBorder(BorderFactory.createEmptyBorder());

		layout.createGridBagConstraints(this.countLabel, 0, 0, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, 0, 0);
		layout.createGridBagConstraints(this.count, 1, 0, 2, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.valuesPanel0, 0, 1, 3, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1, 1, 0, 50);
		layout.createGridBagConstraints(this.sortDirectionLabel, 0, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 0, 0, 0, 0);
		layout.createGridBagConstraints(this.sortDirectionAscending, 1, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.sortDirectionDescending, 2, 2, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.resultPanel, 0, 3, 3, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, 1, 1, 0, 50);
		layout.createGridBagConstraints(this.sort, 0, 4, 3, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);
		layout.createGridBagConstraints(this.copyright, 0, 5, 3, 1, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, 1, 0, 0, 0);

		this.count.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent event) {
				UserInterface.this.updateInputFields();
			}
		});

		this.sort.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				UserInterface.this.run();
			}
		});

		this.copyright.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				Browser.open("http://www.reneknipschild.net/");
			}
			public void mousePressed(MouseEvent e) {

			}
			public void mouseReleased(MouseEvent e) {
				
			}
			public void mouseEntered(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.HAND_CURSOR));
			}
			public void mouseExited(MouseEvent e) {
				UserInterface.this.copyright.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}			
		});
		
		this.sortDirection.add(this.sortDirectionAscending);
		this.sortDirection.add(this.sortDirectionDescending);
		this.frame.add(this.countLabel);
		this.frame.add(this.count);
		this.frame.add(this.valuesPanel0);
		this.frame.add(this.sortDirectionLabel);
		this.frame.add(this.sortDirectionAscending);
		this.frame.add(this.sortDirectionDescending);
		this.frame.add(this.resultPanel);
		this.frame.add(this.sort);
		this.frame.add(this.copyright);

		this.window.add(this.frame, BorderLayout.CENTER);
		this.window.pack();
		this.window.setResizable(true);
		this.window.setLocationRelativeTo(null);
		this.window.setVisible(true);
	}

	private void updateInputFields() {
		int count = Integer.parseInt(this.count.getValue().toString());
		this.values = new JSpinner[count];
		this.valuesPanel2 = new JPanel(new GridLayout(count, 1));
		for (int i = 0; i < this.values.length; i++) {
			this.values[i] = new JSpinner(new SpinnerNumberModel(0.0, -999999999.9, 999999999.9, 0.1));
			this.valuesPanel2.add(this.values[i]);
		}
		this.valuesPanel1.removeAll();
		this.valuesPanel1.add(this.valuesPanel2, BorderLayout.PAGE_START);
		this.valuesPanel1.updateUI();
	}

	private void run() {
		int count = Integer.parseInt(this.count.getValue().toString());
		double[] values = new double[count];
		int mode = 0;
		String[] sorted = new String[count];
		for (int i = 0; i < count; i++) {
			values[i] = Double.parseDouble(this.values[i].getValue().toString());
		}
		if (this.sortDirectionAscending.isSelected()) {
			mode = DoubleSorter.SORT_ASCENDING;
		} else if (this.sortDirectionDescending.isSelected()) {
			mode = DoubleSorter.SORT_DESCENDING;
		}
		values = DoubleSorter.sort(values, mode);
		for (int i = 0; i < values.length; i++) {
			sorted[i] = String.valueOf(values[i]);
		}
		this.result = new JList(sorted);
		this.resultPanel.removeAll();
		this.resultPanel.add(new JScrollPane(this.result, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
		this.resultPanel.updateUI();
	}
}